rule = '<td><div class="tooltip">$rule<span class="tooltiptext">$tooltip</span></div></td>';
rules = {
	importedNoAction:  rule.replace("$rule", "NO ACTION").replace("$tooltip", "do not allow delete/update of primary key if it has been imported"),
	importedKeyCascade: rule.replace("$rule", "CASCADE").replace("$tooltip", "delete/update rows that import a deleted/updated key"),
	importedKeySetNull: rule.replace("$rule", "SET NULL").replace("$tooltip", "change imported key to NULL if its primary key has been deleted/updated"),
	importedKeySetDefault: rule.replace("$rule", "SET DEFAULT").replace("$tooltip", "change imported key to default if its primary key has been deleted/updated"),
	importedKeyRestrict: rule.replace("$rule", "NO ACTION").replace("$tooltip", "same as importedKeyNoAction (for ODBC 2.x compatibility)")
};

imageDiv16 = '<div class="tooltip"><img src="resources/$path.png" alt="$tip" style="vertical-align: middle" width="16" height="16"><span class="tooltiptext">$tip</span></div>';
imageDiv23 = '<div class="tooltip"><img src="resources/$path.png" alt="$tip" style="vertical-align: middle" width="23" height="16"><span class="tooltiptext">$tip</span></div>';
const PRIMARYKEY = imageDiv16.replace("$path", "pk").replaceAll("$tip", "Primary Key");
const FOREIGNKEY = imageDiv16.replace("$path", "fk").replaceAll("$tip", "Foreign Key");
const CHECKED = imageDiv16.replace("$path", "checked").replaceAll("$tip", "Is nullable");
const UNCHECKED = imageDiv16.replace("$path", "unchecked").replaceAll("$tip", "Is not nullable");
const UNIQUE = imageDiv16.replace("$path", "checked").replaceAll("$tip", "Is unique");
const NOT_UNIQUE = imageDiv16.replace("$path", "unchecked").replaceAll("$tip", "Is not unique");
const UNIQUE_ID_PK = imageDiv16.replace("$path", "pk").replaceAll("$tip", "Unique Id / Primary Key");
const COLUMN_IMG = '<img src="resources/column.png" width="16" height="16">';
const CONSTRAINT_IMG = '<img src="resources/constraint.png" width="16" height="16">';
const INDEX_IMG = '<img src="resources/index.png" width="16" height="16">';

relationIcons = {
	ONETOMANY_BI: imageDiv23.replace("$path", "1nbi").replaceAll("$tip", "Bidirectional one-to-many relation"),
	ONETOMANY_UNI: imageDiv23.replace("$path", "1nuni").replaceAll("$tip", "Unidirectional one-to-many relation"),
	ONETOONE_BI: imageDiv23.replace("$path", "11bi").replaceAll("$tip", "Bidirectional one-to-one relation"),
	ONETOONE_UNI: imageDiv23.replace("$path", "11uni").replaceAll("$tip", "Unidirectional one-to-one relation"),
	MANYTOONE_BI: imageDiv23.replace("$path", "n1bi").replaceAll("$tip", "Bidirectional many-to-one relation"),
	MANYTOONE_UNI: imageDiv23.replace("$path", "n1uni").replaceAll("$tip", "Unidirectional many-to-one relation"),
	MANYTOMANY_BI: imageDiv23.replace("$path", "nmbi").replaceAll("$tip", "Bidirectional many-to-many relation"),
	MANYTOMANY_UNI: imageDiv23.replace("$path", "nmuni").replaceAll("$tip", "Unidirectional many-to-many relation"),
	MAP_UNI: imageDiv23.replace("$path", "map").replaceAll("$tip", "Unidirectional one-to-many map relation"),
	ELEMENTCOLLECTION_UNI: imageDiv23.replace("$path", "ec").replaceAll("$tip", "Unidirectional one-to-many element collection relation")
};

var searchItems = [];
var activeTab = -1;
var APP = {};
var MAP = {};
var SVG = {};

var	tablesList;
var	viewsList;
var	seqList;
var	embedsList;
var	pusList;
var	domainObjectsList;
var domainsList;
var	entsList;
var	supersList;
var	enumsList;
var idgenList;
var allDbList;
var allJpaList;
var diagramsList;


function writeTR(title, value) {
	if (value) {
		return "<tr><td nowrap><label>" + title + "</label></td><td>" + value + "</td></tr>";
	} else {
		return "";
	}
}

function writeTRBool(title, value) {
	if (value = true) {
		return "<tr><td nowrap><label>" + title + "</label></td><td>" + value + "</td></tr>";
	} else {
		return "";
	}
}

function shortName(name) {
	pos = name.lastIndexOf(".");
	if(pos > 0) {
		return name.substring(pos + 1); 
	} else {
		return name;
	}
}

function hashId(hcAndTabId) {
	pos = hcAndTabId.indexOf("-");
	if(pos > 0) {
		return hcAndTabId.substring(0, pos);
	} else {
		return hcAndTabId;
	}
}

function tabId(hcAndTabId) {
	pos = hcAndTabId.indexOf("-");
	if(pos > 0) {
		return hcAndTabId.substring(pos + 1);
	} else {
		return "";
	}
}

function toHtmlEnumDetails(hc, run) {
	en = MAP[run]['jp' + hc][0];
	diagram = '';
	if(SVG[run]['pjp' + hc]) {
		diagram = SVG[run]['pjp' + hc];
	}
	
	description = "";
	if(en.do) {
		description = MAP[run]['ds' + en.do];
	}
	
	values = "";
	for (i = 0; i < en.at.length; i++) {
		descr = "";
		if(en.at[i].do) {
			descr = MAP[run]['ds' + en.at[i].do]; 
		}	
		
		values += '<tr class="gradient">' + 
			'<td>' + en.at[i].na + '</td>' + 
			'<td>' + en.at[i].or + '</td>' + 
			'<td>' + descr + '</td>' + 
			'</tr>';
	}
	
	template =
			'<div class="panelContent" id="de-it-jp-' + hc + '">' + 
			'<table>' +
			'<tr class="th-ent" style="border: 1px solid #666;"><th class="fett" colspan="2">Enum</th></tr>' +
			'<tr><td><label>Name:</label></td><td>' + en.na + '</td></tr>' +
			'<tr style="vertical-align: text-top;"><td><label>Description:</label></td><td>' + description + '</td></tr>' +
			'</table>' +
			'<br><br><br>' + 
			'<table class="border">' + 
			'<tr class="th-ent"><th class="fett" colspan="3">Values</th></tr>' +
			'<tr class="th-ent" style="border-bottom: 1px solid #666;">' +
			'<th>Name</th>' +
			'<th>Ordinal</th>' +
			'<th>Description</th>' +
			'</tr>' +
			values +
			'</table>' + 
			'</div>';
			
	template =
		'<div class="flex-container">' +
			'<div class="flex-child">'+
				template+
			'</div>'+
			'<div class="flex-child">'+
				diagram+
			'</div>'+
		'</div>';
			
	return template;
}

function toHtmlJpaDetails(hcAndTabId, panel, run) {
	hc = hashId(hcAndTabId);
	tbId = tabId(hcAndTabId);
	activeTab = -1;

	objArray = MAP[run]['jp' + hc];
	diagram = '';
	if(SVG[run]['pjp' + hc]) {
		diagram = SVG[run]['pjp' + hc];
	}
	
	ulist = "";
	if(objArray.length > 1) {
		ulist = '<ul>';
	}
		
	template = "";
	for (k = 0; k < objArray.length; k++) {
		obj = objArray[k];
		
		tabLabelTyp = "";				
		jpaType = "JPA Entity";
		label = "Super class of";
		if(obj.ty == 'EMBEDDABLE') {
			jpaType = "JPA Embeddable";
			label = "Embedded in";
			tabLabelTyp = "in ";
		} else if(obj.ty == 'MAPPEDSUPERCLASS') {
			jpaType = "JPA Mapped Super Class";
			tabLabelTyp = "of ";
		} else if(obj.ty == 'ENUM') {
			return toHtmlEnumDetails(hc, run);
		}

		tabsId = "";
		if(objArray.length > 1) {
			tabOf = ""; 
			if(obj.ow && obj.ow.length == 1) {
				hcid = hashId(obj.to);
				ow = MAP[run]['jp' + hcid][0];
				tabOf = shortName(ow.na);	
			}
			
			if(obj.oo == true) {
				if(tabOf.length > 0) tabOf += ' ';
				tabOf += '(';
				pulist="";
				for (i = 0; i < obj.pu.length; i++) {
					pu = MAP[run]['pu' + obj.pu[i]];
					pulist += ', ' + pu.na;
				}
				tabOf += pulist.substring(2) + ')'
			}

			tabLabel = "Other";
			if(tabOf.length > 0) {
				tabLabel = tabLabelTyp + tabOf;
			}
			
			tabsId = ' id="tabs-' + k + '"';
			ulist += '<li><a href="#tabs-' + k + '">' + tabLabel;
			ulist += '</a></li>';
			
			if(tbId == obj.ky) {
				activeTab = k;
			}
		}
	
		hierarchy = '<a class="link link1" data-id="it-jp-' + hc + '">' + obj.na + '</a>';
		if (obj.hi) {
			for (i = 1; i < obj.hi.length; i++) {
				hcid = hashId(obj.hi[i]);
				hi = MAP[run]['jp' + hcid][0];
				hierarchy += ' &rarr; <a class="link link1" data-id="it-jp-' + obj.hi[i] + '">' + hi.na + '</a>';	
			}
		}
	
		owner = "";
		if (obj.ow) {
			for (i = 0; i < obj.ow.length; i++) {
				hcid = hashId(obj.ow[i]);
				ow = MAP[run]['jp' + hcid][0];
				owner += ', <a class="link link1" data-id="it-jp-' + obj.ow[i] + '">' + shortName(ow.na) + '</a>';
			}
			owner = "<tr><td nowrap><label>" + label + "</label></td><td>" + owner.substring(2) + "</td></tr>";
		}
	
		persUnits = "";
		if (obj.pu) {
			for (i = 0; i < obj.pu.length; i++) {
				pu = MAP[run]['pu' + obj.pu[i]];
				persUnits += ', <a class="link link1" data-id="it-pu-' + obj.pu[i] + '">' + pu.na + '</a>';
			}
			if(persUnits.length > 0) {
				persUnits = persUnits.substring(2);
			}
			persUnits = "<tr><td nowrap><label>Persistence Units</label></td><td>" + persUnits + "</td></tr>";
		}
	
		table = "";
		Object.keys(obj.ta).forEach(function(name) {
			if(obj.ta[name] != "%") {
				db = MAP[run]['db' + obj.ta[name]];
				table += ', <a class="link link1" data-id="it-db-' + obj.ta[name] + '">' + db.na + '</a>';
			} else {
				table += ", " + name + " (missing table)";
			}
		}); 
		if(table.length > 0) {
			table = table.substring(2);
		}
		table = '<tr><td><label>Table</label></td><td>' + table + '</td></tr>';
	
		description = "";
		if(obj.do) {
			description = MAP[run]['ds' + obj.do];
		}
	
		discriminator = "";
		if(obj.di) {
			discriminator = "<tr><td nowrap><label>Discriminator Value</label></td><td>" + obj.di + "</td></tr>";
		}
	
		attributes = "";
		for (i = 0; i < obj.at.length; i++) {
			
			currentCss = "";
			style = "";
			padding = "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";
			
			if(obj.at[i].cs) {
				style = 'style="display:none;"';
	
				if(obj.at[i].cs == "superClass") {
					currentCss = "superClass";
				} else {
					for (j = 0; j < obj.at[i].de - 1; j++) {
						padding += '&nbsp;&nbsp;&nbsp;';
					}
					padding += '&#8627;&nbsp;';
	
					pos = obj.at[i].cs.indexOf(" ");
					if (pos > 0) {
						first = obj.at[i].cs.substring(0, pos);
						currentCss = " show-" + first + " hide-" + first + obj.at[i].cs.substring(pos);
					} else {
						currentCss = " show-" + obj.at[i].cs + " hide-" + obj.at[i].cs;
					}
				}
			}
			
			// name
			if(obj.at[i].na == '#DiscriminatorColumn' || obj.at[i].ty.ty == '#Id') {
				name = "-";
			} else {
				name = obj.at[i].na;
				if(obj.at[i].su) {
					hcid = hashId(obj.at[i].su);
					superClass =  MAP[run]['jp' + hcid][0];
					superClassName = shortName(superClass.na);
					name = '(<a class="link link1" data-id="it-jp-' + obj.at[i].su + '">' + superClassName + '</a>) ' + name;
				}
			
				if(obj.at[i].em) {
					embedClass = obj.at[i].em + "-" + obj.at[i].na;
					pmCss = "topshow-" + embedClass + " tophide-" + embedClass;
					if (obj.at[i].de > 0) {
						pmCss = pmCss + " tophide-" + obj.at[i].cs;
					}
	 
					pl = "";
					for (j = 0; j < obj.at[i].de; j++) {
						pl += "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";
					}
	
					padding = pl + '<a class="link plusminus ' + pmCss + '" data-klapp="' + embedClass + '">&plusb;</a>&nbsp;';
	
				}
			}
			name = padding + name;
			
			if(obj.at[i].na == '#DiscriminatorColumn') {
				type = "[Technical Descriptor]";
			} else if(obj.at[i].ty.ty == '#Id') {
				type = "[Inherited ID]";
			} else {
				typeObj = obj.at[i].ty;
				if(typeObj.id) {
					hcid = hashId(typeObj.id);
					jclass = MAP[run]['jp' + hcid][0];
					type = '<a class="link link1" data-id="it-jp-' + typeObj.id + '">' + shortName(jclass.na) + '</a>';
				} else {
					type = typeObj.ty;
					if (typeObj.vty || typeObj.vid) {
						type += "&lt;";
						if (typeObj.kid) {
							hcid = hashId(typeObj.kid);
							keyclass = MAP[run]['jp' + hcid][0];
							type += '<a class="link link1" data-id="it-jp-' + typeObj.kid + '">' + shortName(keyclass.na) + '</a>, ';
						} else if (typeObj.kty) {
							type += typeObj.kty + ", "; 
						}
						if (typeObj.vid) {
							hcid = hashId(typeObj.vid);
							valclass = MAP[run]['jp' + hcid][0];
							type += '<a class="link link1" data-id="it-jp-' + typeObj.vid + '">' + shortName(valclass.na) + '</a>';
						} else {
							type += typeObj.vty; 
						}
						type += "&gt;";
					}
				}
			}
			
			props = "";
			if(obj.at[i].pp) {
				propObj = obj.at[i].pp;
				if(propObj.ge) {
					props += "&diamondsuit; ID</br>";
					if(!obj.at[i].em) {
						props += "&diamondsuit; " + propObj.ge;
						if(propObj.gi) {
							props += ' ID Generator: <a class="link link1" data-id="it-ge-' + propObj.gi + '">' + MAP[run]['ge' + propObj.gi].na + '</a>';
						} else {
							props += ' ID Generation';
						}
						props += "</br>";
					} 
					if(obj.at[i].na == "#Id") {
						props += "&diamondsuit;  Replicated from super class</br>";
					}
				}
				if(propObj.iv == true) {
					props += "&diamondsuit; Version</br>";
				}
				if(propObj.et) {
					props += "&diamondsuit; Enum as	" + propObj.et + "</br>";
				}		
				if(propObj.tt) {
					props += "&diamondsuit; Temporal as	" + propObj.tt + "</br>";
				}		
				if(obj.at[i].na == '#DiscriminatorColumn') {
					props += "&diamondsuit; Discriminator values: " + propObj.di + "</br>";
				}
				if(propObj.rt) {
					props += "&diamondsuit; Relation: " + propObj.rt + "</br>";
				}		
				if(propObj.ma) {
					props += "&diamondsuit; Mapped by: " + propObj.ma + "</br>";
				}		
				if(propObj.fe) {
					props += "&diamondsuit; Fetch type: " + propObj.fe + "</br>";
				}		
				if(propObj.ca) {
					props += "&diamondsuit; Cascading: " + propObj.ca + "</br>";
				}		
				if(propObj.no == true) {
					props += "&diamondsuit; Not optional</br>";
				}	
			}	
			
			doc = "";
			if(obj.at[i].do) {
				doc = MAP[run]['ds' + obj.at[i].do];
				pos = doc.indexOf(".");
				if (pos >= 0 && doc.length > pos + 1) {
					// long description
					titleName = shortName(obj.na) + "." + obj.at[i].na;
					doc = doc.substring(0, pos + 1) + ' <a class="popup link" data-id="' + titleName + '|' + obj.at[i].do + '">more...</a>';
				}
			}

			tableColumns = "";
			if(obj.at[i].co) {
				for (j = 0; j < obj.at[i].co.length; j++) {
					tcol = obj.at[i].co[j];
					tableColumns += '<br>';
					if(tcol.ty) {
						tableColumns += tcol.ty + ': '; 
					}
					if(tcol.ti) {
						coltab =  MAP[run]['db' + tcol.ti];
						tableColumns += '<a class="link link1" data-id="it-db-' + tcol.ti + '">' + coltab.na + '</a>.';				 
					} else if(tcol.tn) {
						tableColumns += tcol.tn + '.';
					}
					
					tableColumns += tcol.na;
					if(tcol.cn) {
						tableColumns += '<br><div style="font-size: smaller">(references ';
						if(tcol.ri) {
							reftab =  MAP[run]['db' + tcol.ri];
							tableColumns += '<a class="link link1" data-id="it-db-' + tcol.ri + '">' + reftab.na + '</a>.';				 
						} else if(tcol.rn) {
							tableColumns += tcol.rn + '.';				 
						} 
						 
						tableColumns += tcol.cn;	
						tableColumns += ')</div>';
					}
				}
				if(tableColumns.length > 0) {
					tableColumns = tableColumns.substring(4);
				}
			}
			
			tableLink = "";
			domLink = "";
			columnRef = "";
			if(APP[run].db.me) {
				Object.keys(obj.at[i].li).forEach(function(colId) {
					columnRef += ',{"ci":"' + colId + '","ti":"' + obj.at[i].li[colId] + '"}';
				}); 
				if (columnRef.length > 0) {
					columnRef = columnRef.substring(1);
					columnRef = ' data-columns=\'[' + columnRef + ']\'';
					tableLink = '<a class="link jpa2dblink">' + 
						'<img src="resources/link-y.png" alt="Link Column" style="vertical-align: middle" width="16" height="16"></a>';
				}
			}
			if(obj.at[i].od && obj.at[i].na != '#DiscriminatorColumn' && obj.at[i].ty.ty != '#Id') {
					columnRef += ' data-jClass="' + obj.at[i].od +'" data-attr="' + obj.at[i].id +'" ';
					domLink = '<a class="link jpa2domlink">' + 
						'<img src="resources/link-r.png" alt="Link Domain Attribute" style="vertical-align: middle" width="16" height="16"></a>';
			}
			
			attributes += '<tr' + columnRef + ' class="' + obj.at[i].id + ' gradient ' + currentCss + '" ' + style + '>' +
				'<td style="border-right: 0">' + tableLink + '</td>' +
				'<td style="border-right: 0">' + domLink + '</td>' +
				'<td>' + name  + '</td>' + 
				'<td>' + type +'</td>' + 
				'<td>' + props + '</td>' + 
				'<td>' + doc + '</td>' +
				'<td>' + tableColumns + '</td>' + 
				'</tr>';
		}
		
		template +=
				'<div class="panelContent"' + tabsId + '>' +
				'<table>' + 
				'<tr class="th-ent border"><th class="fett" colspan="2">' + jpaType + '</th></tr>' + 
				'<tr><td><label>Name</label></td><td>' + hierarchy + '</td></tr>' +
				owner +
				persUnits +
				table +
				'<tr style="vertical-align: text-top;"><td><label>Description</label></td><td>' + description + '</td></tr>' +
				discriminator +
				'</table>' + 
				'<br><br><br>' + 
				'<table class="border">' + 
				'<tr class="th-ent"><th class="fett" colspan="7">Class Attributes</th></tr>' + 
				'<tr class="th-ent" style="border-bottom: 1px solid #666;">' + 
				'<th colspan="3">Name</th>' + 
				'<th>Type</th>' + 
				'<th>Properties</th>' + 
				'<th>Description</th>' + 
				'<th>Table/Column</th>' +
				'</tr>' + 
				attributes + 
				'</table>' +
				'</div>';
	}
	
	if(ulist.length > 0) {
		script = '<script>$( function() {$( ".tabs-' + panel + '" ).tabs({active:' + activeTab + '});} );</script>';
		template = script + '<div class="tabs-' + panel + '">' + ulist + '</ul>' + template + '</div>';
	}
	template =
		'<div class="flex-container">' +
			'<div class="flex-child">'+
				template+
			'</div>'+
			'<div class="flex-child">'+
				diagram+
			'</div>'+
		'</div>';

	return template;
}

function toHtmlGeDetails(hc, run) {
	gen = MAP[run]['ge' + hc];
	
	enss = '';
	for (i = 0; i < gen.en.length; i++) {
		entity = MAP[run]['jp' + gen.en[i]][0]; 
		enss += ', <a class="link link1" data-id="it-jp-' + gen.en[i] + '">' + shortName(entity.na)  + '</a>';
	}
	
	if(enss.length > 0) {
		enss = '<tr><td><label>ID Generation for </label></td><td>' + enss.substring(2) + '</td></tr>';
	}
	
	seq = '';
	if(gen.si) {
		seq = '<tr><td><label>Sequence</label></td><td><a class="link link1" data-id="it-seq-' + gen.si + '">' + gen.sn  + '</a></td></tr>';
	}
	
	tabl = '';
	if(gen.ti) {
		tabl =  '<tr><td><label>Table</label></td><td><a class="link link1" data-id="it-db-' + gen.ti + '">' + gen.ta  + '</a></td></tr>';
		tabl += '<tr><td><label>Primary Key Column</label></td><td>' + gen.cn  + '</td></tr>';
		tabl += '<tr><td><label>Primary Key Value</label></td><td>' + gen.cv  + '</td></tr>';
		tabl += '<tr><td><label>Value Column</label></td><td>' + gen.vc  + '</td></tr>';
	}

	template = 
			'<div class="panelContent" id="de-it-ge-' + hc + '">'+
		    '<table>'+ 
			'<tr class="th-ent" style="border: 1px solid #666;"><th class="fett" colspan="2">ID Generator ' + gen.na + '</th></tr>' +
			'<tr><td><label>Name</label></td><td>' + gen.na + '</td></tr>' +
			'<tr><td><label>Type</label></td><td>' + gen.ty + '</td></tr>' +		    
			'<tr><td><label>Allocation Size</label></td><td>' + gen.ad + '</td></tr>' +		    
			'<tr><td><label>Initial Value</label></td><td>' + gen.iv + '</td></tr>' +	
			seq +	   
			tabl + 
			enss +	    
			'</table>' +		
			'</div>';

	return template;	
}

function toHtmlSeqDetails(hc, run) {
	seq = MAP[run]['se' + hc];
	
	gis = '';
	for (i = 0; i < seq.gi.length; i++) {
		generator = MAP[run]['ge' + seq.gi[i]]; 
		gis += ', <a class="link link1" data-id="it-ge-' + seq.gi[i] + '">' + generator.na + '</a>';
	}
	
	if(gis.length > 0) {
		gis = '<tr><td><label>Sequence Generators</label></td><td>' + gis.substring(2) + '</td></tr>';
	}

	template = 
			'<div class="panelContent" id="de-it-seq-' + hc + '">'+
		    '<table>'+ 
			'<tr class="tbHead" style="border: 1px solid #666;"><th class="fett" colspan="2">Sequence ' + seq.na + '</th></tr>' +
			'<tr><td><label>Name</label></td><td>' + seq.na + '</td></tr>' +
			'<tr><td><label>Start Value</label></td><td>' + seq.st + '</td></tr>' +		    
			'<tr><td><label>Min Value</label></td><td>' + seq.mi + '</td></tr>' +		    
			'<tr><td><label>Max Value</label></td><td>' + seq.ma + '</td></tr>' +		    
			'<tr><td><label>Increment Value</label></td><td>' + seq.in + '</td></tr>' +	
			gis +	    
			'</table>' +		
			'</div>';

	return template;	
}

function toHtmlDbDetails(hc, run) {
	db = MAP[run]['db' + hc];
	diagram = '';
	if(SVG[run]['adb' + hc]) {
		diagram = SVG[run]['adb' + hc];
	}

	mappedEnt = "";
	var hcid;
	if(db.me) {
		for (i = 0; i < db.me.length; i++) {
			hcid = hashId(db.me[i]);
			jpa = MAP[run]['jp' + hcid][0];
			mappedEnt += '<a class="link link1" data-id="it-jp-' + db.me[i] + '">' + jpa.na + '</a><br>';
		}
		if(mappedEnt != '') {
			mappedEnt = '<tr><td valign="top"><label>Mapped Entities</label></td><td>' + mappedEnt + '</td></tr>';
		}
	}

	assEnt = "";
	if(db.ae) {
		for (i = 0; i < db.ae.length; i++) {
			hcaenid = hashId(db.ae[i]);
			jpa = MAP[run]['jp' + hcaenid][0];
			assEnt += '<a class="link link1" data-id="it-jp-' + db.ae[i] + '">' + jpa.na + '</a><br>';
		}
		if(assEnt != '') {
			assEnt = '<tr><td valign="top"><label>Associated Entities</label></td><td>' + assEnt + '</td></tr>';
		}
	}

	gis = '';
	if(db.gi)
	for (i = 0; i < db.gi.length; i++) {
		generator = MAP[run]['ge' + db.gi[i]]; 
		gis += ', <a class="link link1" data-id="it-ge-' + db.gi[i] + '">' + generator.na + '</a>';
	}
	
	if(gis.length > 0) {
		gis = '<tr><td><label>Table Generators</label></td><td>' + gis.substring(2) + '</td></tr>';
	}

	rem = "";
	if(db.re) {
		rem = '<tr style="vertical-align: text-top;"><td><label>Description</label></td><td>' + db.re + '</td></tr>';
	}
	
	showDefaults = "";
	if(db.de) {
		showDefaults = '<th>Default</th>';
	}

	showRemarks = "";
	if(db.cr) {
		showRemarks = '<th>Remarks</th>';
	}
	
	columns = "";
	for (i = 0; i < db.co.length; i++) {

		icon = "";		
		if(db.co[i].pk == true) {
			icon = PRIMARYKEY;
		}

		if(db.co[i].fk == true) {
			if(icon != '') {
				icon += " ";
			}
			icon += FOREIGNKEY;
		}
		
		defValue = "";
		if(db.de) {
			if(db.co[i].df) {
				defValue = '<td>' + db.co[i].df + '</td>';
			} else {
				defValue = '<td></td>';
			}
		}

		remark = "";
		if(db.cr) {
			if(db.co[i].re) {
				remark = '<td>' + db.co[i].re + '</td>';
			} else {
				remark = '<td></td>';
			}
		}
		
		nullable = CHECKED;

		if(db.co[i].nu == false) {
			nullable = UNCHECKED;
		}
		
		jpaLink = "";
		domLink = "";
		columnRef = "";
		if(db.co[i].li) {
			columnRef = db.co[i].li.map(function(item) {
				return JSON.stringify(item);
			});
			columnRef = ' data-jpalink=\'[' + columnRef + ']\'';
			jpaLink = '<a class="link db2jpalink">' + 
				'<img src="resources/link-b.png" alt="Link Jpa Attribute" style="vertical-align: middle" width="16" height="16"></a>';

			if(db.me && typeof db.co[i].dc == 'undefined') {	
				domLink = '<a class="link db2domlink">' + 
					'<img src="resources/link-r.png" alt="Link Domain Attribute" style="vertical-align: middle" width="16" height="16"></a>';
			}
		}

		columns += '<tr' + columnRef + ' class="gradient" id="' + db.co[i].id + '">' + 
			'<td style="border-right: 0">' + jpaLink + '</td>' +
			'<td style="border-right: 0">' + domLink + '</td>' +
			'<td style="border-right: 0">' + icon + '</td>' +
			'<td>' + db.co[i].na + '</td>' +
			'<td>' + db.co[i].ty + '</td>' +
			'<td>' + db.co[i].si + '</td>' +
			defValue +
			remark + 
			'<td>' + nullable + '</td>' +  
			'</tr>';
	}
	
	indexes = "";
	for (i = 0; i < db.in.length; i++) {
		unique = UNIQUE;
		if(db.in[i].un == false) {
			unique = NOT_UNIQUE;
		}

		indexes += '<tr class="gradient">' + 
			'<td>' + db.in[i].co + '</td>' +
			'<td>' + db.in[i].na + '</td>' +
			'<td>' + unique + '</td>' +
			'</tr>';
	}

	constraints = "";
	for (i = 0; i < db.fk.length; i++) {
		img = PRIMARYKEY;
		table = "";
		type = "Primary Key";
		fkCol = "";
		delrule="<td></td>";
		updrule="<td></td>";
		if(db.fk[i].ta) {
			img = FOREIGNKEY;
			table = '<a class="link link1" data-id="it-db-' + db.fk[i].ta + '">' + MAP[run]['db' + db.fk[i].ta].na + '</a>';
			type = "Foreign Key";
			fkCol = db.fk[i].fc;
			delrule=rules[db.fk[i].dr];
			updrule=rules[db.fk[i].ur];
		}
		
		constraints += '<tr class="gradient">' + 
			'<td style="border-right: 0">' + img + '</td>' + 
			'<td>' + db.fk[i].co + '</td>' + 
			'<td>' + db.fk[i].na + '</td>' +
			'<td>' + table + '</td>' +
			'<td>' + fkCol + '</td>' +
			delrule + 
			updrule + 
			'</tr>';
	}

	template = 
			'<div class="panelContent" id="de-it-db-' + hc + '">' + 
			'<table>' +
			'<tr class="tbHead" style="border: 1px solid #666;"><th class="fett" colspan="2">Database ' + db.ty + '</th></tr>' +
			'<tr><td><label>Name</label></td><td>' + db.na + '</td></tr>' +
			'<tr><td><label>Type</label></td><td>' + db.la + '</td></tr>' +
			mappedEnt +
			assEnt +
			gis +
			rem +
			'</table>' +
			'<br><br><br>' + 
			'<table class="border">' + 
			'<tr class="tbHead"><th class="fett" colspan="' + (db.cs + 2) + '">Columns</th></tr>'+ 
			'<tr class="tbHead" style="border-bottom: 1px solid #666;">'+ 
			'<th colspan="4">Name</th>' +
			'<th>Type</th>' +
		    '<th>Size</th>' +
			showDefaults +
			showRemarks +
			'<th>Nullable</th>'+
			'</tr>'+ 
			columns + 
			'</table>' + 
			'<br><br><br>' + 
			'<table class="border">' + 
			'<tr class="tbHead"><th class="fett" colspan="3">Indexes</th></tr>' + 
			'<tr class="tbHead" style="border-bottom: 1px solid #666;">' + 
			'<th>Columns</th>' +
			'<th>Name</th>' +
			'<th>Unique</th>' +
			'</tr>' + 
			indexes + 
			'</table>' + 			
			'<br><br><br>' + 
			'<table class="border">' + 
			'<tr class="tbHead"><th class="fett" colspan="7">Constraints</th></tr>' + 
			'<tr class="tbHead" style="border-bottom: 1px solid #666;">' + 
		    '<th colspan="2">Column</th>' +
			'<th>Name</th>' +
			'<th>PK Table</th>' + 
			'<th>PK Column</th>' +
			'<th>Delete Rule</th>' + 
			'<th>Update Rule</th>' +
			'</tr>' + 
			constraints + 
			'</table>' + 
			'</div>';

	template =
		'<div class="flex-container">' +
		'<div class="flex-child">'+
		template+
		'</div>'+
		'<div class="flex-child">'+
		diagram+
		'</div>'+
		'</div>';

	return template;
}

function toHtmlPuDetails(hc, run) {
	pu = MAP[run]['pu' + hc];
	pro = "";
	if(pu.pr) {
		Object.keys(pu.pr).forEach(function(key){
			pro = pro + "<tr><td nowrap>" +	key + "</td><td>" + pu.pr[key] + "</td></tr>\n";
		}); 
	}
	
	mapFiles = "";
	if(pu.mf) {
		Object.keys(pu.mf).forEach(function(key){
			title = "ORM mapping file " + key;
			mapFiles += ', ' + ' <a class="popup link" data-id="' + title + "|" + pu.mf[key] + '">' + key + '</a>';
		}); 
	}
	if(mapFiles.length > 0) {
		mapFiles = mapFiles.substring(2);
	}
	
	perty = "";
	for (i = 0; i < pu.mt.length; i++) {
		if (pu.mt[i].id) {
			hcid = hashId(pu.mt[i].id);
			jpa = MAP[run]['jp' + hcid][0];
			perty = perty + '<tr><td nowrap><a class="link link1" data-id="it-jp-' + pu.mt[i].id +'">' + jpa.na + "</a></td><td>" + pu.mt[i].so + "</td></tr>\n";
		} else {
			perty = perty + "<tr><td nowrap>" +	pu.mt[i].na + "</td><td>" + pu.mt[i].so + "</td></tr>\n";	
		}
	}

	template =  
			'<div class="panelContent" id="de-it-pu-' + hc + '">' + 
			'<table>' + 
			'<tr class="th-ent border"><th class="fett" colspan="2">Persistence Unit</th></tr>' + 
			writeTR("Name", pu.na) +
			writeTR("Descriptor file", pu.ur) +
			writeTR("Provider", pu.pv) +
			writeTR("Transaction Type", pu.tx) +
			writeTR(pu.dt, pu.dn) +
			writeTR("Mapping Files", mapFiles) +
			writeTR("Jar Files", pu.jf) +
			writeTR("Classes", pu.cl) +
			writeTRBool("Exclude unlisted classes", pu.ex) +
			writeTR("Shared Cache Mode", pu.sh) +
			writeTR("Validation Mode", pu.va) +
			writeTR("Description", pu.de) +
			'</table>' + 
			'<br><br><br>' + 
			'<table class="border">' + 
			'<tr class="th-ent"><th class="fett" colspan="2">Properties</th></tr>' + 
			'<tr class="th-ent" style="border-bottom: 1px solid #666;"><th>Name</th><th>Value</th></tr>' + 
			pro + 
			'</table>' + 
			'<br><br><br>' + 
			'<table class="border">' + 
			'<tr class="th-ent"><th class="fett" colspan="2">Effective Managed Persistent Types</th></tr>' + 
			'<tr class="th-ent" style="border-bottom: 1px solid #666;"><th>Name</th><th>Source</th></tr>' + 
			perty + 
			'</table>' + 
			'</div>';
	return template;
}
	
function toHtmlDomainDetails(hc, run) {
	dom = MAP[run]['dd' + hc];
	
	packages = "";
	for (i = 0; i < dom.pa.length; i++) {
		packages += " ; " + dom.pa[i];
	}
	if (packages.length > 0) {
		packages = packages.substring(3);
	}
	
	rows = "";
	for (i = 0; i < dom.do.length; i++) {
		jpa = MAP[run]['jp' + dom.do[i]][0];
		domObj = '<a class="link link1" data-id="it-do-' + dom.do[i] + '">' + shortName(jpa.na).toUpperCase() + '</a>';
		rows += '<tr class="gradient"><td style="border-right: 0">' + domObj + '</td></tr>';
	}
	
	template = 
			'<div class="panelContent" id="de-it-dd-' + hc + '">'+
		    '<table>'+ 
			'<tr class="th-dom border"><th class="fett" colspan="2">Domain</th></tr>'+ 
			'<tr><td><label>Name</label></td><td>' + dom.na + '</td></tr>'+
			'<tr><td><label>Packages</label></td><td>' + packages + '</td></tr>'+
			'</table>' +		
			'<br><br><br>' +
			'<table class="border">' +
			'<tr class="th-dom"><th class="fett">Domain Objects</th></tr>' +
			rows +
			'</table>' +
			'</div>';

	return template;	
}
	
function toHtmlDoDetails(hc, run) {
	hcid = hashId(hc);
	dom = MAP[run]['do' + hcid];
	jpa = MAP[run]['jp' + hcid][0];
	domain = MAP[run]['dd' + dom.bc];
	
	diagram = '';
	if(SVG[run]['doo' + hcid]) {
		diagram = SVG[run]['doo' + hcid];
	}

	name = shortName(jpa.na).toUpperCase();
	entity = '<a class="link link1" data-id="it-jp-' + hc + '">' + jpa.na + '</a>';
	
	table = "";
	Object.keys(jpa.ta).forEach(function(name) {
		if(jpa.ta[name] != "%") {
			db = MAP[run]['db' + jpa.ta[name]];
			table += ', <a class="link link1" data-id="it-db-' + jpa.ta[name] + '">' + db.na + '</a>';
		} else {
			table += ", " + name + " (missing table)";
		}
	}); 
	if(table.length > 0) {
		table = table.substring(2);
	}
	
	description = "";
	if(jpa.do) {
		description = MAP[run]['ds' + jpa.do];
	}

	rows = "";
	for (i = 0; i < dom.at.length; i++) {
		primKeyIcon="";
		if (dom.at[i].pp && dom.at[i].pp.ii == true) {
			primKeyIcon = UNIQUE_ID_PK;
		}
		
		relKeyIcon="";
		if (dom.at[i].pp && dom.at[i].pp.re) {
			relKeyIcon = relationIcons[dom.at[i].pp.re]; 
		}
		
		typob = dom.at[i].ty;
		type = "";
		if (typob.ty) {
			type = typob.ty;
			if(typob.ty.indexOf(" of ") > 0) {
				type += "(";
			}
		}
		
		if (typob.kty) {
			type += typob.kty + ", ";
		}
		if(typob.kid) {
			hcid = hashId(typob.kid);
			hck = MAP[run]['jp' + hcid][0];
			keyname = shortName(hck.na);
			if(typob.dik == 'do') {
				keyname = keyname.toUpperCase();
			}
			type += '<a class="link link1" data-id="it-' + typob.dik + '-' + typob.kid + '">' + keyname + '</a>, ';
		}
		
		if (typob.vty) {
			type += typob.vty;
		}
		if(typob.vid) {
			hcid = hashId(typob.vid);
			hcv = MAP[run]['jp' + hcid][0];
			valname = shortName(hcv.na);
			if(typob.div == 'do') {
				valname = valname.toUpperCase();
			}
			type += '<a class="link link1" data-id="it-' + typob.div + '-' + typob.vid + '">' + valname + '</a>';
		}
		
		if(typob.ty && typob.ty.indexOf(" of ") > 0) {
			type += ")";
		}

		rdescr = "";
		if(dom.at[i].do) {
			rdescr = MAP[run]['ds' + dom.at[i].do];
			pos = rdescr.indexOf(".");
			if (pos >= 0 && rdescr.length > pos + 1) {
				// long description
				titleName = name + "." + dom.at[i].na;
				rdescr = rdescr.substring(0, pos + 1) + ' <a class="popup link" data-id="' + titleName + '/' + dom.at[i].do + '">more...</a>';
			}
		}

		entLink = "";
		tableLink = "";
		columnRef = columnRef = ' data-jClass="' + dom.at[i].od +'" data-attr="' + dom.at[i].id +'" ';
		entLink = '<a class="link dom2jpalink">' + 
			'<img src="resources/link-b.png" alt="Link JPA Attribute" style="vertical-align: middle" width="16" height="16"></a>';

		tableLinkId = "";
		columnLinkId = "";
		jpaclassId = hashId(dom.at[i].od);
		jpaObj = MAP[run]['jp' + jpaclassId];
		var json = "";
		for (m = 0; m < jpaObj.length; m++) {
			jpa = jpaObj[m];
				
			for (k = 0; k < jpa.at.length; k++) {
				if(jpa.at[k].na.toUpperCase() == dom.at[i].na) {
					Object.keys(jpa.at[k].li).forEach(function(colId) {
						json = ',{"ci":"' + colId + '","ti":"' + jpa.at[k].li[colId] + '"}';
					}); 
					if (json.length > 0) {
						json = json.substring(1);
					}
					columnRef += ' data-columns=\'[' + json + ']\'';
					break;
				}
			}
			if(json.length > 0) {
				break;
			}
		}
			
		if(json.length > 0) {
			tableLink = '<a class="link dom2dblink">' + 
			'<img src="resources/link-y.png" alt="Link Column" style="vertical-align: middle" width="16" height="16"></a>';
		}

		rows += '<tr' + columnRef + ' class="' + dom.at[i].id + ' gradient">' 
			+ '<td style="border-right: 0">' + entLink + '</td>'
			+ '<td style="border-right: 0">' + tableLink + '</td>'
			+ '<td style="border-right: 0">' + primKeyIcon + '</td>' 
			+ '<td>' + dom.at[i].na + '</td>' 
			+ '<td style="border-right: 0">' + relKeyIcon + '</td>' 
			+ '<td>' + type + '</td>' 
			+ '<td>' + rdescr + '</td></tr>';
	}

	template = 
			'<div class="panelContent" id="de-it-do-' + hc + '">'+
		    '<table>'+ 
			'<tr class="th-dom border"><th class="fett" colspan="2">Business Domain Type</th></tr>'+ 
			'<tr><td><label>Name</label></td><td>' + name + '</td></tr>'+
			'<tr><td><label>Domain</label></td><td>' + domain.na + '</td></tr>'+
			'<tr><td><label>Entity</label></td><td>' + entity + '</td></tr>'+
			'<tr><td><label>Table</label></td><td>' + table + '</td></tr>'+
			'<tr style="vertical-align: text-top;"><td><label>Description</label></td><td>' + description + '</td></tr>'+
			'</table>' +		
			'<br><br><br>' +
			'<table class="border">' +
			'<tr class="th-dom"><th class="fett" colspan="7">Attributes</th></tr>' +
			'<tr class="th-dom" style="border-bottom: 1px solid #666;">' +
			'<th colspan="2"></th>' +
			'<th colspan="2">Name</th>' + 
			'<th colspan="2">Type</th>' + 
			'<th>Description</th></tr>' +
			rows +
			'</table>' +
			'</div>';
			
	template =
		'<div class="flex-container">' +
			'<div class="flex-child">'+
				template+
			'</div>'+
			'<div class="flex-child">'+
				diagram+
			'</div>'+
		'</div>';
			
	return template;
}

function toHtmlErrorDetails(run, issueType) {
	template = '';
	if(issueType == "Issue Report") {
		if (typeof(APP[run].er) != 'undefined') {
			template += toHtmlErrorDetails2(run, APP[run].er, issueType);
		}
		if (typeof(APP[run].eri) != 'undefined' && APP[run].eri.length > 0) {
			template += '<br><br><a class="link link_ignoredErrors">Show ignored issues</a><br>';
		}
		
	} else if (issueType == "Ignored Issues") {
		if (typeof(APP[run].eri) != 'undefined') {
			template += toHtmlErrorDetails2(run, APP[run].eri, issueType);
			template += '<br><br><a class="link link_ignoredErrors">Hide ignored issues</a><br>';
		}
	}

	return template;
}

function toHtmlErrorDetails2(run, objList, issueType) {
	rows = "";
	for (i = 0; i < objList.length; i++) {
		curER = objList[i];
		text = curER.te;
		
		j = 0;
		Object.keys(curER.pa).forEach(function(key) {
			if(curER.pa[key]) {
				typ = curER.pa[key].substring(0, 2);
				ref = curER.pa[key].substring(2);
				rep = '<a class="link link1" data-id="it-' + typ + '-' + ref + '">' + key + '</a>';
				text = text.replace("{" + j + "}", rep);
				
			} else {
				text = text.replace("{" + j + "}", key);
			}
			j++;
		}); 

		rows += '<tr class=" gradient">' 
			+ '<td>' + curER.nr + '</td>'
			+ '<td>' + APP[run].erd['a' + curER.id].na + '</td>'
			+ '<td>' + APP[run].erd['a' + curER.id].le + '</td>'
			+ '<td>' + text + '</td>'
			+ '<td><a class="popup2 link errDet" data-id="ERR' + i + '">' 
			+ '<img src="resources/zoom-in.png" alt="Details" style="vertical-align: middle"></a></td></tr>';
	}
	
	template = 
			'<div class="panelContent">'+
			'<br>' +
			'<table class="border">' +
			'<tr class="th-err"><th class="fett" colspan="5">' + issueType + '</th></tr>' +
			'<tr class="th-err" style="border-bottom: 1px solid #666;">' +
			'<th>ID</th>' +
			'<th>Type</th>' +
			'<th>Level</th>' + 
			'<th>Message</th>' + 
			'<th>Details</th>' +
			rows +
			'</table>' +
			'</div>';
				
	return template;
}

function toHtmlListings(run) {
	var	tables = [];
	var	views = [];
	var	seq = [];
	var	embeds = [];
	var	pus = [];
	var	domainObjects = [];
	var	domains = [];
	var	ents = [];
	var	supers = [];
	var	enums = [];
	var idgens = [];
	var allDb = [];
	var allJpa = [];
	var allDiagr = [];

	tablesPos = 0;
	viewsPos = 0;
	seqPos = 0;
	embedsPos = 0;
	pusPos = 0;
	domainObjectsPos = 0;
	domainsPos = 0;
	supersPos = 0;
	enumsPos = 0;
	idgenPos = 0;
	entsPos = 0;
	allDbPos = 0;
	allJpaPos = 0;
	allDiagrPos = 0;
	
	for (let mapId in MAP[run]) {
		mapObj = MAP[run][mapId];
		
		if (mapId.startsWith('db')) {
			allDb[allDbPos++] = {id: mapId.substring(2), na: mapObj.na};
			if(mapObj.ty == 'Table') {
				tables[tablesPos++] = {id: mapId.substring(2), na: mapObj.na};	
			} else {
				views[viewsPos++] = {id: mapId.substring(2), na: mapObj.na};
			}
			
		} else if (mapId.startsWith('se')) {
			allDb[allDbPos++] = {id: mapId.substring(2), na: mapObj.na};
			seq[seqPos++] = {id: mapId.substring(2), na: mapObj.na};
			
		} else if (mapId.startsWith('ge')) {
			allJpa[allJpaPos++] = {id: mapId.substring(2), na: mapObj.na};
			idgens[idgenPos++] = {id: mapId.substring(2), na: mapObj.na};
				
		} else if (mapId.startsWith('jp')) {
			allJpa[allJpaPos++] = {id: mapId.substring(2), na: shortName(mapObj[0].na)};
			if(mapObj[0].ty == 'EMBEDDABLE') {
				embeds[embedsPos++] = {id: mapId.substring(2), na: shortName(mapObj[0].na)};	
			} else if(mapObj[0].ty == 'ENUM') {
				enums[enumsPos++] = {id: mapId.substring(2), na: shortName(mapObj[0].na)};	
			} else if(mapObj[0].ty == 'ENTITY') {
				ents[entsPos++] = {id: mapId.substring(2), na: shortName(mapObj[0].na)};	
			} else if(mapObj[0].ty == 'MAPPEDSUPERCLASS') {
				supers[supersPos++] = {id: mapId.substring(2), na: shortName(mapObj[0].na)};	
			}
			
		} else if (mapId.startsWith('pu')) {
			pus[pusPos++] = {id: mapId.substring(2), na: mapObj.na};	
			
		} else if (mapId.startsWith('do')) {
			jpa = MAP[run]['jp' + mapId.substring(2)][0];
			domainObjects[domainObjectsPos++] = {id: mapId.substring(2), na: shortName(jpa.na).toUpperCase()};
		} else if (mapId.startsWith('dd')) {
			jpa = MAP[run]['jp' + mapId.substring(2)][0];
			domains[domainsPos++] = {id: mapId.substring(2), na: mapObj.na};
		}
	}
	
	for(let svgId in SVG[run]) {
		svgObj = SVG[run][svgId];
		if (svgId.startsWith('xen')) {
			allDiagr[allDiagrPos++] = {id: 'xen', na: 'Entities', pos: 1};	
		} else if (svgId.startsWith('xta')) {
			allDiagr[allDiagrPos++] = {id: 'xta', na: 'Tables', pos: 2};
		} else if (svgId.startsWith('xdo')) {
			allDiagr[allDiagrPos++] = {id: 'xdo', na: 'Domains and Domain Objects', pos: 3};
		} else if (svgId.startsWith('xet')) {
			allDiagr[allDiagrPos++] = {id: 'xet', na: 'Entity/Table Mappings', pos: 4};
		} else if (svgId.startsWith('xdt')) {
			allDiagr[allDiagrPos++] = {id: 'xdt', na: 'Domain Object/Table Mappings', pos: 5};
		} else if (svgId.startsWith('xde')) {
			allDiagr[allDiagrPos++] = {id: 'xde', na: 'Domain Object/Entity Mappings', pos: 6};
		}
	}
	
	// sort
	allDb.sort(function(a,b){
		if (a.na < b.na) {return -1;}
  		if (a.na > b.na) {return 1;}
  		return 0;
	});

	allJpa.sort(function(a,b){
		if (a.na < b.na) {return -1;}
  		if (a.na > b.na) {return 1;}
  		return 0;
	});

	allDiagr.sort(function(a,b){
		if (a.pos < b.pos) {return -1;}
  		if (a.pos > b.pos) {return 1;}
  		return 0;
	});

	domains.sort(function(a,b){
		if (a.na < b.na) {return -1;}
  		if (a.na > b.na) {return 1;}
  		return 0;
	});

	// counts
	$("#tableCount").text(" (" + tablesPos + ")");
	$("#viewCount").text(" (" + viewsPos + ")");
	$("#seqCount").text(" (" + seqPos + ")");
	$("#embeddableCount").text(" (" + embedsPos + ")");
	$("#superClassCount").text(" (" + supersPos + ")");
	$("#enumCount").text(" (" + enumsPos + ")");
	$("#idgenCount").text(" (" + idgenPos + ")");
	$("#entityCount").text(" (" + entsPos + ")");
	$("#allJpaCount").text(" (" + allJpaPos + ")");
	$("#allDbCount").text(" (" + allDbPos + ")");
	$("#puCount").text(" (" + pusPos + ")");
	$("#domCount").html("&nbsp;(" + domainObjectsPos + ")");
	
	if (tables.length > 0) {
		tablesList = '<ul id="t-ul">';
		for(j = 0 ; j < tables.length ; j++) {
			tablesList += '<li><a class="peritems" id="it-db-' + tables[j].id + '">' + tables[j].na + '</a></li>';
		}
		tablesList += '</ul>';
	} else {
		tablesList = '';
	}
	
	if (views.length > 0) {
		viewsList = '<ul id="t-ul">';
		for(j = 0 ; j < views.length ; j++) {
			viewsList += '<li><a class="peritems" id="it-db-' + views[j].id + '">' + views[j].na + '</a></li>';
		}
		viewsList += '</ul>';
	} else {
		viewsList = '';
	}

	if (seq.length > 0) {
		seqList = '<ul id="t-ul">';
		for(j = 0 ; j < seq.length ; j++) {
			seqList += '<li><a class="peritems" id="it-seq-' + seq[j].id + '">' + seq[j].na + '</a></li>';
		}
		seqList += '</ul>';
	} else {
		seqList = '';
	}

	if (embeds.length > 0) {
		embedsList = '<ul id="t-ul">';
		for(j = 0 ; j < embeds.length ; j++) {
			embedsList += '<li><a class="peritems" id="it-jp-' + embeds[j].id + '">' + embeds[j].na + '</a></li>';
		}
		embedsList += '</ul>';
	} else {
		embedsList = '';
	}
	
	if (pus.length > 0) {
		pusList = '<ul id="t-ul">';
		for(j = 0 ; j < pus.length ; j++) {
			pusList += '<li><a class="peritems" id="it-pu-' + pus[j].id + '">' + pus[j].na + '</a></li>';
		}
		pusList += '</ul>';
	} else {
		pusList = '';
	}

	if (idgens.length > 0) {
		idgenList = '<ul id="t-ul">';
		for(j = 0 ; j < idgens.length ; j++) {
			idgenList += '<li><a class="peritems" id="it-ge-' + idgens[j].id + '">' + idgens[j].na + '</a></li>';
		}
		idgenList += '</ul>';
	} else {
		idgenList = '';
	}
	
	if (enums.length > 0) {
		enumsList = '<ul id="t-ul">';
		for(j = 0 ; j < enums.length ; j++) {
			enumsList += '<li><a class="peritems" id="it-jp-' + enums[j].id + '">' + enums[j].na + '</a></li>';
		}
		enumsList += '</ul>';
	} else {
		enumsList = '';
	}

	if (ents.length > 0) {
		entsList = '<ul id="t-ul">';
		for(j = 0 ; j < ents.length ; j++) {
			entsList += '<li><a class="peritems" id="it-jp-' + ents[j].id + '">' + ents[j].na + '</a></li>';
		}
		entsList += '</ul>';
	} else {
		entsList = '';
	}
	
	if (supers.length > 0) {
		supersList = '<ul id="t-ul">';
		for(j = 0 ; j < supers.length ; j++) {
			supersList += '<li><a class="peritems" id="it-jp-' + supers[j].id + '">' + supers[j].na + '</a></li>';
		}
		supersList += '</ul>';
	} else {
		supersList = '';
	}
	
	if (domainObjects.length > 0) {
		domainObjectsList = '<ul id="t-ul">';
		for(j = 0 ; j < domainObjects.length ; j++) {
			domainObjectsList += '<li><a class="peritems" id="it-do-' + domainObjects[j].id + '">' + domainObjects[j].na + '</a></li>';
		}
		domainObjectsList += '</ul>';
	} else {
		domainObjectsList = '';
	}

	if (domains.length > 0) {
		domainsList = '<ul id="t-ul">';
		for(j = 0 ; j < domains.length ; j++) {
			domainsList += '<li><a class="peritems" id="it-dd-' + domains[j].id + '">' + domains[j].na + '</a></li>';
		}
		domainsList += '</ul>';
	} else {
		domainsList = '';
	}

	if (allDb.length > 0) {
		allDbList = '<ul id="t-ul">';
		for(j = 0 ; j < allDb.length ; j++) {
			allDbList += '<li><a class="peritems" id="it-db-' + allDb[j].id + '">' + allDb[j].na + '</a></li>';
		}
		allDbList += '</ul>';
	} else {
		allDbList = '';
	}

	if (allJpa.length > 0) {
		allJpaList = '<ul id="t-ul">';
		for(j = 0 ; j < allJpa.length ; j++) {
			allJpaList += '<li><a class="peritems" id="it-jp-' + allJpa[j].id + '">' + allJpa[j].na + '</a></li>';
		}
		allJpaList += '</ul>';
	} else {
		allJpaList = '';
	}
	
	if (allDiagr.length > 0) {
		diagramsList = '<ul id="t-ul">';
		for(j = 0 ; j < allDiagr.length ; j++) {
			diagramsList += '<li><a class="peritems" id="it-di-' + allDiagr[j].id + '">' + allDiagr[j].na + '</a></li>';
		}
		diagramsList += '</ul>';
	} else {
		diagramsList = '';
	}
}

function scrollHorizontal(panel) {
	panel.scrollLeft("1000000");
    var scrollWidth = panel.prop("scrollWidth");
    var diff = (scrollWidth-panel.scrollLeft()) / 2;
    var middle = scrollWidth / 2 - diff;
    panel.scrollLeft(middle);
}

function scrollVertical(panel) {
    panel.scrollTop("1000000");//scroll to max
    var scrollHeight = panel.prop("scrollHeight");
    var diff = (scrollHeight - panel.scrollTop()) / 2;
    var middle = scrollHeight / 2 - diff;
    panel.scrollTop(middle);
}

function scrollToMiddle(panelId, tabId, ui) {
	if (ui.newPanel[0].getAttribute("id") == tabId) {
		panel = "panel" + panelId;
		scrollVertical($("#" + panel));
		scrollHorizontal($("#" + panel));
	}
}

function scrollToMiddle(panel) {
	scrollVertical($("#" + panel));
	scrollHorizontal($("#" + panel));
}

function toDatabaseDetails(run) {
	if(typeof(APP[run].db.me) == 'undefined') {
		return 'Database has not been analyzed';
	}
	
	det = '';
	if(typeof(SVG[run].xen) == 'undefined' && APP[run].do) {
		det = '<table class="border">' +
			'<tr class="tbHead" style="border-bottom: 1px solid #666;"><th class="fett">Database Description</th></tr>' +
			'<td>' + APP[run].do + '</td>' +
			'</table>' +
			'<br><br>';
	}

	det += '<p><table class="border">';
	det += '<tr class="tbHead" style="border-bottom: 1px solid #666;"><th class="fett" colspan="2">Database Properties</th></tr>';
	det += '<tr class="gradient"><td><label>Catalog</label></td><td>' + APP[run].db.me.cat + '</td></tr>';
	det += '<tr class="gradient"><td><label>Schema</label></td><td>' + APP[run].db.me.sch + '</td></tr>';
	det += '<tr class="gradient"><td><label>Url</label></td><td>' + APP[run].db.me.url + '</td></tr>';
	det += '<tr class="gradient"><td><label>User</label></td><td>' + APP[run].db.me.use + '</td></tr>';
	det += '<tr class="gradient"><td><label>Name</label></td><td>' + APP[run].db.me.prn + '</td></tr>';
	det += '<tr class="gradient"><td><label>Version</label></td><td>' + APP[run].db.me.prv + '</td></tr>';
	det += '<tr class="gradient"><td><label>Driver</label></td><td>' + APP[run].db.me.drn + '</td></tr>';
	det += '<tr class="gradient"><td><label>Driver Version</label></td><td>' + APP[run].db.me.drv + '</td></tr>';
	if(APP[run].db.me.mcc != '0') {
		det += '<tr class="gradient"><td><label>Max concurrent connections</label></td><td>' + APP[run].db.me.mcc + '</td></tr>';
	}
	det += '</td></tr></table></p>';
	return det;
}	

function toApplicationDetails(run) {
	if(typeof(SVG[run].xen) == 'undefined') {
		return 'java sources have not been analyzed';
	}

	doc = '';
	if (APP[run].do) {
		doc = '<table class="border">' +
			'<tr class="th-ent"><th class="fett" >Application Description</th></tr>' +
			'<td>' + APP[run].do + '</td>' +
			'</table>' +
			'<br><br>';
	}

	rows = '';
	for (i = 0; i < APP[run].so.length; i++) {
		rows += '<tr class="gradient"><td>' + APP[run].so[i].so + '</td>';
		if(APP[run].so[i].na) {
			rows += '<td>' + APP[run].so[i].na + '&nbsp;&nbsp;<button id="soubut-' + i + '" class="link1" data-id="it-souman-' + i + '">Show</button></td>';
			rows += '<td id="sou-' + i + '">' + '</td>';
		} else {
			rows += '<td></td><td></td>';			
		}
		
		rows += '</tr>';	
	}
	
	deps = '';
	for (i = 0; i < APP[run].de.length; i++) {
		deps += '<tr class="gradient"><td>' + APP[run].de[i].de + '</td>';
		deps += '<td>' + APP[run].de[i].so + '</td>';
		if(APP[run].de[i].ma) {
			deps += '<td>' + APP[run].de[i].ma.na + '&nbsp;&nbsp;<button id="depbut-' + i + '" class="link1" data-id="it-depman-' + i + '">Show</button></td>';
			deps += '<td id="manif-' + i + '">' + '</td>';
		} else {
			deps += '<td></td><td></td>';			
		}
		
		deps += '</tr>';	
	}
	
	template = 
		'<div class="panelContent">'+
		doc +
		'<table class="border">' +
		'<tr class="th-ent"><th class="fett" colspan="3">Analyzed Sources</th></tr>' +
		'<tr class="th-ent" style="border-bottom: 1px solid #666;">' +
		'<th>Source</th>' + 
		'<th>Manifest</th>' +
		'<th>Content</th>' +
		'</tr>' +
		rows +
		'</table>' +
		
		'<br><br>' +
		'<table class="border">' +
		'<tr class="th-ent"><th class="fett" colspan="4">Dependencies</th></tr>' +
		'<tr class="th-ent" style="border-bottom: 1px solid #666;">' +
		'<th>Dependency</th>' + 
		'<th>Origin</th>' +
		'<th>Manifest</th>' +
		'<th>Content</th>' +
		'</tr>' +
		deps +
		'</table>' +
		'</div>';
			
	template =
		'<div class="flex-container">' +
			'<div class="flex-child">' +
				template +
			'</div>' +
			'<div class="flex-child">' +
				'<div class="manifest"></div>' +
			'</div>' +
		'</div>';
			
	return template;
}

function toManifestDetails(key, key1, run) {
	det = '<p><table class="border"><tr class="th-ent" style="border-bottom: 1px solid #666;"><th class="fett" colspan="2">';

	var manif;
	if (key1 == 'DEP') {
		manif = APP[run].de[key].ma;
	} else {
		manif = APP[run].so[key];
	}
	
	det += manif.na;
	det += '</th></tr>';
		
	if(manif.att) {
		Object.keys(manif.att).forEach(function(key) {
			det += '<tr class="gradient"><td><label>';
			det += key;
			det += '</label></td><td class="mfval">';
			det += 	manif.att[key];
			det += '</td></tr>';
		}); 
	}
		
	det += '</table></p></br>';
	return det;
}

function toCompareDetails(run) {
	cols = 2 + Object.keys(TABLEDIFF[0].v).length;
	jobsTitle = '';
	for (prop in TABLEDIFF[0].v) {
		jobsTitle += '<th>' + prop + '</th>';
	}

	rows = '';
	for (i = 1; i < TABLEDIFF.length; i++) {
		
		jobValues = '';
		for (prop in TABLEDIFF[i].v) {
			value = TABLEDIFF[i].v[prop];
			if (TABLEDIFF[i].d == 'ADD_REM') {
				if (TABLEDIFF[i].v[prop].length > 0) {
					value = '<img src="resources/check.png" width="16" height="16">';	
				} else {
					value = '<img src="resources/cancel.png" width="16" height="16">';
				}
			} 
			
			jobValues += '<td>' + value + '</td>';
		}
		
		label = TABLEDIFF[i].t;
		var typePrefix = "Table ";		
		id = TABLEDIFF[i].id[run.substring(1)];
		if (TABLEDIFF[i].ot == "Table") {
			if(id) {
				label = '<a class="link link1" data-id="it-db-' + id + '">' + TABLEDIFF[i].t + '</a>';
			}
			
			if(TABLEDIFF[i].p) {
				if(TABLEDIFF[i].pt.startsWith('column')) {
					typePrefix = "Column ";
				} else if(TABLEDIFF[i].pt.startsWith('constraint')) {
					typePrefix = "Constraint ";
				} else if(TABLEDIFF[i].pt.startsWith('index')) {
					typePrefix = "Index ";
				}
			}

		} else {
			if(id) {
				label = '<a class="link link1" data-id="it-seq-' + id + '">' + TABLEDIFF[i].t + '</a>';
			}
			typePrefix = "Sequence ";
		}
		
		property = "";
		if(TABLEDIFF[i].p) {
			property = "." + TABLEDIFF[i].p;
		}
		 
		rows += '<tr class="gradient">'
		+ '<td>' + typePrefix + label + property + '</td>'
		+ '<td>' + TABLEDIFF[i].a + '</td>'
		+ jobValues
		+ '</tr>';
	}
		
	tableTemplate = 
			'<div class="panelContent" id="compare">'+
			'<br>' +
			'<table class="border">' +
			'<tr class="th-err"><th class="fett" colspan="' + cols + '">Database Comparison</th></tr>' +
			'<tr class="th-err" style="border-bottom: 1px solid #666;">' +
			'<th>Type</th>' +
			'<th>Modified Property</th>' + 
			jobsTitle + 
			'</tr>' +
			rows +
			'</table>' +
			'</div><br><br>';

	// class differences
	cols = 3 + Object.keys(CLASSDIFF[0].v).length;
	jobsTitle = '';
	for (prop in CLASSDIFF[0].v) {
		jobsTitle += '<th>' + prop + '</th>';
	}

	rows = '';
	for (i = 1; i < CLASSDIFF.length; i++) {
		jobValues = '';
		for (prop in CLASSDIFF[i].v) {
			value = CLASSDIFF[i].v[prop];
			if (CLASSDIFF[i].d == 'ADD_REM') {
				if (CLASSDIFF[i].v[prop].length > 0) {
					value = '<img src="resources/check.png" width="16" height="16">';	
				} else {
					value = '<img src="resources/cancel.png" width="16" height="16">';
				}
			} 
			
			jobValues += '<td>' + value + '</td>';
		}
		
		label = CLASSDIFF[i].t;
		var typePrefix = "";	
		id = CLASSDIFF[i].id[run.substring(1)];
		if (CLASSDIFF[i].ot == "ClassModel") {
			if(id) {
				label = '<a class="link link1" data-id="it-jp-' + id + '">' + CLASSDIFF[i].t + '</a>';
			}
			
		} else {
			typePrefix = "ID Generator ";
			if(id) {
				label = '<a class="link link1" data-id="it-ge-' + id + '">' + CLASSDIFF[i].t + '</a>';
			}
		}
		
		rows += '<tr class="gradient">'
		+ '<td>' + typePrefix + label + '</td>'
		+ '<td>' + CLASSDIFF[i].p + '</td>'
		+ '<td>' + CLASSDIFF[i].a + '</td>'
		+ jobValues
		+ '</tr>';
	}
		
	classTemplate = 
			'<div class="panelContent" id="compare">'+
			'<br>' +
			'<table class="border">' +
			'<tr class="th-err"><th class="fett" colspan="' + cols + '">Java Comparison</th></tr>' +
			'<tr class="th-err" style="border-bottom: 1px solid #666;">' +
			'<th>Type</th>' +
			'<th>Attribute</th>' +
			'<th>Modified Property</th>' + 
			jobsTitle + 
			'</tr>' +
			rows +
			'</table>' +
			'</div><br><br>';
			
	return tableTemplate + classTemplate;
}
